% Ginters Buss 2011
%Translation from Wildi R code DFA analytic
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% if useful, cite Ginters Buss "Real-time signal extraction with
% regularized multivariate direct filter approach",
% in Journal of Forecasting
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%This function computes analytic DFA-solutions
%K_dfa are the number of frequency ordinates (default: (length of time series)/2)
%L_dfa is the length of the MA,
%weight_func_dfa is a weighting function in the frequency domain (the periodogram in the DFA)
%lambda_dfa is a hyperparameter which emphasizes phase artifacts
%Gamma_dfa is the transferfunction of the symmetric filter (target)
%Lag_dfa is the lag-parameter: Lag_dfa=0 implies real-time filtering, Lag_dfa=L_dfa/2 implies symmetric filter
%expweight_dfa: for lambda_dfa=0 and expweight_dfa=0 the mean-square filter is obtained
%cutoff_dfa frequency (in radians, like cutoff_dfa=%pi/7): high frequency noise with frequencies larger than cutoff_dfa are magnified if expweight_dfa>0; 
%if i1_dfa<-T, then Gamma_dfa^(0)=Gamma_dfa(0) i.e. transfer functions of asymmetric and symmetric filters are identical in frequency zero (first order constraint)
%if i2_dfa<-T then time-shift of asymmetric filter vanishes in freq zero
%Don't set i2_dfa<-T if Lag_dfa>0 or if Gamma_dfa is not a lowpass filter
%The function returns the weights of the MA as well as the Transferfunction

function [b,trffkt]=dfa(K,L,lambda,weight_func,Lag,Gamma,expweight,cutoff,i1,i2)
    omega_Gamma=fix(cutoff*K/pi);
    if (K-omega_Gamma+1)>0
        weight_h=weight_func.*[ones(1,omega_Gamma),(1:(K-omega_Gamma+1)).^expweight]';
    else
        weight_h=weight_func.*ones(1,K+1)';
    end
    % First order filter restriction: assigning a `large' weight to frequency zero
    if i1
        weight_h(1)=max(1.d+10,weight_h(1));
    end
    X=exp(1i*Lag*pi*[0:K]/K)'.*ones(1,K+1)'.*sqrt(weight_h);
    X_y=(exp(1i*Lag*pi*[0:K]/K).*ones(1,K+1))';
    % Second order restriction: time shift in frequency zero vanishes
    if i2
        for l=2:L-1
            X=[X,(cos((l-1-Lag)*pi*[0:K]/K)'-((l-1)/(L-1))*cos((L-1-Lag)*pi*[0:K]/K)'+(sqrt(1+Gamma*lambda)*1i).*(sin((l-1-Lag)*pi*[0:K]/K)'-((l-1)/(L-1))*sin((L-1-Lag)*pi*[0:K]/K)')).*sqrt(weight_h)];
            X_y=[X_y,cos((l-1-Lag)*pi*[0:K]/K)'-((l-1)/(L-1))*cos((L-1-Lag)*pi*[0:K]/K)'+1i*(sin((l-1-Lag)*pi*[0:K]'/K)-((l-1)/(L-1))*sin((L-1-Lag)*pi*[0:K]'/K))];
        end
        xtx=real(X)'*real(X)+imag(X)'*imag(X);
        % MA filter weights
        b=(inv(xtx)*real(X_y)'*(Gamma.*weight_h));
        % the last weight is a function of the previous ones through the second order restriction
        b=[b;-sum(b.*[0:length(b)-1]')/(length(b)+1)];
    else
        for l=2:L
            X=[X,(cos((l-1-Lag)*pi*[0:K]'/K)+(sqrt(1+Gamma*lambda)*1i).*sin((l-1-Lag)*pi*[0:K]'/K)).*sqrt(weight_h)];
            X_y=[X_y,cos((l-1-Lag)*pi*[0:K]'/K)+1i*sin((l-1-Lag)*pi*[0:K]'/K)];
        end
        xtx=real(X)'*real(X)+imag(X)'*imag(X);
        % MA filter weights
        b=(inv(xtx)*real(X_y)'*(Gamma.*weight_h));
    end
    % transfer function; must be -1 in front of 1i; different typing from R
    trffkt=[1:K+1]';
    trffkt(1)=sum(b);
    for k=1:K
        trffkt(k+1)=exp(-1i*k*[0:length(b)-1]*pi/K)*b; 
    end
    % plot(abs(trffkt))


