function ylevf = olsf(y,ylag,xxlag,nfor,begf);
% PURPOSE: estimates ols of order n
%          and produces f-step-ahead forecasts
%-------------------------------------------------------------
% USAGE:yfor = varf(y,ylag,xlag,nfor,begf,x,transf)
% where:    y    = an (nobs * neqs) matrix of data in levels
%                   where 1st column is y and others are x
%           ylag = the y lag length
%           xlag = the x lag length
%           nfor = the forecast horizon
%           begf = the beginning date of the forecast
%                  (defaults to length(x) + 1)            
%-------------------------------------------------------------
% NOTE: constant term included automatically
%-------------------------------------------------------------
% RETURNS: 
%      yfor = an nfor x neqs matrix of level forecasts for each equation
%-------------------------------------------------------------
% SEE ALSO: var, plt_var, prt_var
%-------------------------------------------------------------
% modified by Ginters Buss originally
% written by:
% James P. LeSage, Dept of Economics
% Texas State University-San Marcos
% 601 University Drive
% San Marcos, TX 78666
% jlesage@spatial-econometrics.com


[nobs neqs] = size(y);
% adjust nobs to feed the lags
nmin = min(nobs,begf-1);
% adjust nvar for constant term
nlag=max(ylag,xxlag);

k = ylag+(neqs-1)*xxlag+1;


% generate lagged rhs matrix
xlag = [mlag(y(:,1),ylag),mlag(y(:,2:end),xxlag)];
% constant term
iota = ones(nobs,1);
% truncate to feed lags and to end at begf-1 for estimation
dys  = trimr(y,nlag,nobs-begf+1);
dy   = y;
xlag = trimr(xlag,nlag,nobs-begf+1);
iota = trimr(iota,nlag,nobs-begf+1);

% add x-matrix and constant to x-matrix

xmat = [xlag iota];


% dimension some result matrices
bmat = zeros(k,1);
yfor = zeros(nfor,1);

% ----- get bhat estimates

% save time by computing xpx only once
xpx = xmat'*xmat;

% pull out each y-vector and run regressions
for j=1:neqs;
 yvec = dys(:,j);
 bhat = (xpx)\(xmat'*yvec); 
 % save bhat
 bmat(:,j) = bhat;
end; 
% end of loop over equations


% given bhat estimates, generate future forecasts 
% These may be levels, 1st-differences, growth rates or seas diff's
% we worry transforming back to levels later

% 1-step-ahead forecast 
xtrunc = [dy(nmin-nlag:nmin,:)
          zeros(1,neqs)];
xfor = [mlag(xtrunc(:,1),ylag),mlag(xtrunc(:,2:end),xxlag)];
[xend ~] = size(xfor);
xobs = xfor(xend,:);

xvec = [xobs 1];

for i=1:neqs;
bhat = bmat(:,i);
yfor(1,i) = xvec*bhat;
end;

xnew = zeros(ylag+1,neqs);

% 2 through nlag-step-ahead forecasts
for step=2:ylag

if step <= nfor

xnew(1:ylag-step+1,:) = dy(nmin-ylag+step:nmin,:);
xnew(ylag-step+2:ylag,:) = yfor(1:step-1,:);
xnew(ylag+1,:) = zeros(1,neqs);

xfor = [mlag(xnew(:,1),ylag),mlag(xnew(:,2:end),xxlag)];
[xend ~] = size(xfor);
xobs = xfor(xend,:);

xvec = [xobs 1];



% loop over equations

bhat = bmat(:,1);
yfor(step) = xvec*bhat;


end;

end;

% nlag through nfore-step-ahead forecasts
for step=ylag:nfor-1;

if step <= nfor

cnt = step-(ylag-1);

 for i=1:ylag;
  xnew(i,:) = yfor(cnt,:);
  cnt = cnt+1;
 end;
 
xfor = [mlag(xnew(:,1),ylag),mlag(xnew(:,2:end),xxlag)];
[xend ~] = size(xfor);
xobs = xfor(xend,:);

xvec = [xobs 1];


% loop over equations

bhat = bmat(:,1);
yfor(step+1) = xvec*bhat;

end;

end;

% we now worry about transforming the forecasts back
% to levels

ylevf = yfor;


  
